package hu.stunts.scr.ui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;

import hu.stunts.scr.apps.*;
import hu.stunts.scr.exceptions.NoScoreboardEntriesException;

public class HIGTableComponent extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3226479269734041958L;
	
	private Race race;
	private JTable higTable;
	private JButton bMakeHig;
	
	private boolean correctTimes;

	private class HIGTableModel extends AbstractTableModel {
		/**
		 * 
		 */
		private static final long serialVersionUID = -7179425957708245967L;
		
		private String[] columnNames = {"#",
				"Racer",
				"Car",
				"Opp.",
				"Opp. Car",
		"Time",
		"Trm",
		"Gap",
		"Track"};
		private Object[][] data;

		public int getColumnCount() {
			return columnNames.length;
		}

		public int getRowCount() {
			return data.length;
		}

		public String getColumnName(int col) {
			return columnNames[col];
		}

		public Object getValueAt(int row, int col) {
			return data[row][col];
		}

		public void setValueAt(Object value, int row, int col) {

			data[row][col] = value;
			fireTableCellUpdated(row, col);
		}

	
		HIGTableModel(int rows, int cols){
			data = new Object[rows][cols];
		}
	}
	
	private ActionListener onMakeHig = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			String s = (String)JOptionPane.showInputDialog(
					HIGTableComponent.this,
					"HIG destination path:\n"
					+ "Default: <NAME OF THE TRACK>.HIG",
					"Write to .HIG",
					JOptionPane.PLAIN_MESSAGE, null, null,
					race.getRootDir().getAbsolutePath() + File.separator + race.getRaceTitle() + ".HIG");

			//If a string was returned, say so.
			File f = null;
			if (s != null)
				f = new File(s);
			if (f != null) {
				int cancelFileWriteIndex = 0; //write
				if (f.isDirectory()) {
					JOptionPane.showMessageDialog(HIGTableComponent.this, "Path is a directory. Choose another path.");
					cancelFileWriteIndex = 1; //do not write
				}
				else if (!f.getParentFile().exists()) {
					JOptionPane.showMessageDialog(HIGTableComponent.this, "Destination directory does not exist. Choose another path.");
					cancelFileWriteIndex = 1;
				}
				else if (f.exists()) {
					cancelFileWriteIndex = JOptionPane.showConfirmDialog(HIGTableComponent.this, "File exists. Overwrite?", "Confirm overwrite", JOptionPane.YES_NO_OPTION);
				}
				if (cancelFileWriteIndex == 0) {
					try {
						writeHigFile(f);
					} catch (SecurityException ex) {
						JOptionPane.showMessageDialog(HIGTableComponent.this, "File write permission denied.", "Failed to write file", JOptionPane.ERROR_MESSAGE);
						actionPerformed(e);
					}
				}
				else
					actionPerformed(e);		
			}
			
		}
		
	};	
	
	public HIGTableComponent(Race extRace, boolean correctTimes) throws NoScoreboardEntriesException {
		super();
		this.correctTimes = correctTimes;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));	
		JLabel raceTitle = new JLabel(extRace.getRaceTitle());
		raceTitle.setMaximumSize(new Dimension(640,80));
		raceTitle.setAlignmentX(CENTER_ALIGNMENT);
		try {
			setupTable(extRace, correctTimes);
			JScrollPane scrollPane = new JScrollPane(higTable);
			higTable.setFillsViewportHeight(true);
			add(raceTitle);
			add(scrollPane);
			bMakeHig = new JButton("Write .HIG file");
			bMakeHig.setAlignmentX(CENTER_ALIGNMENT);
			bMakeHig.addActionListener(onMakeHig);
			add(bMakeHig);
		} catch (NoScoreboardEntriesException e) {
			add(new JLabel("No scoreboard entries are available..."));
			throw e;
		}
	}
	
	private void writeHigFile (File f) throws SecurityException {
		try {
			race.generateHIG(f, correctTimes);
		} catch (SecurityException e) {
			throw e;
		}
	}
	
	private void setupTable(Race extRace, boolean correctTimes) throws NoScoreboardEntriesException {
		race = extRace;
		race.loadActiveReplays();
		String[][] raceData = race.generateHIGStrings(correctTimes);
		if (raceData == null || raceData.length == 0)
			throw new NoScoreboardEntriesException();
		HIGTableModel higModel = new HIGTableModel(raceData.length, raceData[0].length + 1);
		for (int i = 0; i < raceData.length; i++) {
			higModel.setValueAt(i + 1, i, 0);
			for (int j = 0; j < raceData[i].length; j++)
				higModel.setValueAt(raceData[i][j], i, j + 1);
		}
		higTable = new JTable(higModel);
		higTable.getColumnModel().getColumn(0).setPreferredWidth(24); //#
		higTable.getColumnModel().getColumn(8).setPreferredWidth(84); //Trk
		higTable.getColumnModel().getColumn(2).setPreferredWidth(168);//Car
		higTable.getColumnModel().getColumn(1).setPreferredWidth(120);//Name
		higTable.getColumnModel().getColumn(3).setPreferredWidth(60); //Opp
		higTable.getColumnModel().getColumn(4).setPreferredWidth(60); //OppCar
		higTable.getColumnModel().getColumn(5).setPreferredWidth(60); //Time
		higTable.getColumnModel().getColumn(6).setPreferredWidth(36); //Trm.
		higTable.getColumnModel().getColumn(7).setPreferredWidth(72); //Gap
	}
		
}
