package hu.stunts.scr.utils;


import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Map;

public class InfoFormatter {

	private final ClassLoader ccl = Thread.currentThread().getContextClassLoader();
	private URL urlToCarNames = ccl.getResource("resources/carnames.txt");
	private URL urlToCarAbbrevs = ccl.getResource("resources/carabbrevs.txt");
	private URL urlToCarColours = ccl.getResource("resources/carcolours.txt");
	
	public static final int
		CLOCK = 0, BERNIE = 1, OTTO = 2,
		JOE = 3, CHERRY = 4, HELEN = 5,
		SKID = 6;
	private Map<String, String> carsDict = null;
	private Map<String, String> abbrDict = null;
	private Map<String, String[]> colourDict = null;

	public InfoFormatter(File currentDir, boolean initializeColours) {
		//cars list
		int dictInitSize = 50;
		try {
			InputStream carsIS = urlToCarNames.openStream();
			carsDict = TxtParsing.simpleMakeMapFromPairs(carsIS, dictInitSize, true);
		} catch (IOException e) {
			e.printStackTrace();
		}
		File extraCarsASCII = new File(currentDir, "racernames.txt");
		if (extraCarsASCII.exists()) {
			Map<String, String> extraNamesDict = TxtParsing.simpleMakeMapFromPairs(extraCarsASCII, dictInitSize, true);
			carsDict.putAll(extraNamesDict);
		}
		//car abbreviations list
		try {
			InputStream abbrIS = urlToCarAbbrevs.openStream();
			abbrDict = TxtParsing.simpleMakeMapFromPairs(abbrIS, dictInitSize, true);
		} catch (IOException e) {
			e.printStackTrace();
		}
		File extraAbbrASCII = new File(currentDir, "carabbrevs.txt");
		if (extraAbbrASCII.exists()) {
			Map<String, String> extraAbbrDict = TxtParsing.simpleMakeMapFromPairs(extraAbbrASCII, dictInitSize, true);
			abbrDict.putAll(extraAbbrDict);
		}
		if (initializeColours) {
			try {
				InputStream coloursIS = urlToCarColours.openStream();
				colourDict = TxtParsing.makeMultiValuedVarLengthMap(coloursIS, dictInitSize, true);
			} catch (IOException e) {
				e.printStackTrace();
			}
			File extraColoursASCII = new File(currentDir, "carcolours.txt");
			if (extraColoursASCII.exists()) {
				Map<String, String[]> extraColourDict = TxtParsing.makeMultiValuedVarLengthMap(extraColoursASCII, dictInitSize, true);
				colourDict.putAll(extraColourDict);
			}
		}
	}
	
	public InfoFormatter(File currentDir) {
		this(currentDir, false);
	}
	
	public InfoFormatter() {
		this(new File(System.getProperty("user.home")));
	}
	
	public static String formatLaptime (String raw) {
		String output = "";
		double time = 0;
		if (raw != null) {
			time = Double.parseDouble(raw);
			if (time < 0) {
				time = Math.abs(time);
				output = output + "-";
			}
			double minutes = Math.floor(time / 60);
			output = output + Integer.toString((int)minutes) + ":";
			time = time - minutes * 60;
			NumberFormat secondsFormat = new DecimalFormat("#00.00");
			output = output + secondsFormat.format(time);
		}
		return output;
	}
	
	public static String formatTimeGap (String raw) {
		String output = "";
		double time = 0;
		if (raw != null) {
			time = Double.parseDouble(raw);
			if (time < 0) {
				time = Math.abs(time);
				output = output + "-";
			}
			else
				output = output + "+";
			double minutes = Math.floor(time / 60);
			if (minutes > 0)
				output = output + Integer.toString((int)minutes) + ":";
			time = time - minutes * 60;
			NumberFormat secondsFormat = new DecimalFormat("#00.00");
			output = output + secondsFormat.format(time);
		}
		return output;
	}
	
	public static String formatOpponentName (int oppIndex) {
		String name = "";
		switch (oppIndex) {
		case (BERNIE) : name = "Bernie Rubber"; break;
		case (OTTO) : name = "Herr Otto Partz"; break;
		case (JOE) : name = "Joe Stallin"; break;
		case (CHERRY) : name = "Cherry Chassis"; break;
		case (HELEN) : name = "Helen Wheels"; break;
		case (SKID) : name = "Skid Vicious"; break;
		}
		return name;
	}
	
	public static String formatOpponentAbbr (int oppIndex) {
		String name = "";
		switch (oppIndex) {
		case (BERNIE) : name = "BR"; break;
		case (OTTO) : name = "OP"; break;
		case (JOE) : name = "JS"; break;
		case (CHERRY) : name = "CC"; break;
		case (HELEN) : name = "HW"; break;
		case (SKID) : name = "SV"; break;
		}
		return name;
	}
	
	//this is not static so that we can delegate dictionary initialization to an instance.
	public String formatCarName (String carID) {
		String name;
		if (carsDict.containsKey(carID))
			name = carsDict.get(carID);
		else
			name = carID;
		return name;
	}
	
	public String formatCarAbbr (String carID) {
		String name;
		if (abbrDict.containsKey(carID))
			name = abbrDict.get(carID);
		else
			name = carID;
		return name;
	}
	
	public String formatCarColour (String carID, int colourIndex) {
		String colour;
		if (colourDict != null && colourDict.containsKey(carID)) {
			String[] colourArray = colourDict.get(carID);
			if (colourArray != null && !(colourIndex < 0) && colourIndex < colourArray.length)
				colour = colourArray[colourIndex];
			else
				colour = Integer.toString(colourIndex);
		}
		else
			colour = Integer.toString(colourIndex);
		return colour;
	}
	
}
